package com.ticketmaster.presence.demo;

import android.content.DialogInterface;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;

import androidx.annotation.ColorInt;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.ticketmaster.presencesdk.PresenceSDK;
import com.ticketmaster.presencesdk.login.PresenceSdkConfigListener;
import com.ticketmaster.presencesdk.login.PresenceSimpleLoginListener;
import com.ticketmaster.presencesdk.login.TMLoginApi;

public class PresenceSdkHostActivity extends AppCompatActivity implements PresenceSdkConfigListener {

    private static final String TAG = PresenceSdkHostActivity.class.getSimpleName();

    private final PresenceSimpleLoginListener mSimpleLoginListener = new PresenceSimpleLoginListener() {

        @Override
        public void onLoginSuccessful(TMLoginApi.BackendName backendName, String accessToken) {
            super.onLoginSuccessful(backendName, accessToken);
            Log.d(TAG, "onLoginSuccessful() called with: backendName = ["
                    + backendName + "], accessToken = [" + accessToken + "]");
        }

        @Override
        public void onLoginCancelled(TMLoginApi.BackendName backendName) {
            super.onLoginCancelled(backendName);
            Log.d(TAG, "onLoginCancelled() called with: backendName = [" + backendName + "]");
        }
    };


    private ConstraintLayout mGettingStartedContainer;
    private AlertDialog mProgressDialog;
    private boolean mConfigSuccessFul;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_psdk_host);
        mGettingStartedContainer = findViewById(R.id.getting_started_container);
        mGettingStartedContainer.setVisibility(View.VISIBLE);

        showLoading();

        // configure experience
        configureExperienceSDK();

        // configure the Presence SDK
        configurePresenceSDK();
    }

    private void showLoading() {
        if (mProgressDialog == null) {
            mProgressDialog = new AlertDialog.Builder(this)
                    .setView(LayoutInflater.from(this).inflate(R.layout.layout_loading_view, null))
                    .setCancelable(false)
                    .create();
            mProgressDialog.setCanceledOnTouchOutside(false);
        }
        mProgressDialog.show();
    }

    private void configurePresenceSDK() {

        // Attach this Activity to receive callbacks for the configuration listener
        PresenceSDK.getPresenceSDK(getApplicationContext()).registerConfigListener(this);

        // Configure your branding color for the SDK
        final int brandingColor = Color.parseColor(BuildConfig.BRANDING_COLOR);
        PresenceSDK.getPresenceSDK(getApplicationContext())
                .setBrandingColors(brandingColor, brandingColor , brandingColor);

        // Set the configuration
        PresenceSDK.getPresenceSDK(getApplicationContext()).setConfig(
                BuildConfig.CONSUMER_KEY, /* Consumer key provided on dev portal */
                BuildConfig.TEAM_NAME, /* your team display name */
                BuildConfig.USE_NAM); /* true for new account manager or false for classic */
    }

    @Override
    public void onPresenceSdkConfigSuccessful() {
        Log.d(TAG, "onPresenceSdkConfigSuccessful() called");
        mConfigSuccessFul = true;
        invalidateOptionsMenu();
        if (mProgressDialog != null) {
            mProgressDialog.dismiss();
        }
        mGettingStartedContainer.setVisibility(View.GONE);

        // unregister the configuration listener
        PresenceSDK.getPresenceSDK(getApplicationContext()).unregisterConfigListener(PresenceSdkHostActivity.this);

        // start the SDK when configuration is successful
        PresenceSDK.getPresenceSDK(getApplicationContext())
                .start(PresenceSdkHostActivity.this, R.id.presenceSDKView, mSimpleLoginListener);
    }

    @Override
    public void onPresenceSdkConfigFailed(String errorMessage) {
        Log.d(TAG, "onPresenceSdkConfigFailed() called with: errorMessage = [" + errorMessage + "]");
        mConfigSuccessFul = false;
        invalidateOptionsMenu();
        if (mProgressDialog != null) {
            mProgressDialog.dismiss();
        }

        // unregister the configuration listener
        PresenceSDK.getPresenceSDK(getApplicationContext()).unregisterConfigListener(PresenceSdkHostActivity.this);

        final AlertDialog.Builder builder = new AlertDialog.Builder(PresenceSdkHostActivity.this)
                .setTitle(R.string.configuration_error_title)
                .setMessage(R.string.configuration_error_message)
                .setPositiveButton(R.string.retry, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        showLoading();
                        configurePresenceSDK();
                    }
                })
                .setNegativeButton(R.string.cancel, null)
                .setCancelable(false);

        final AlertDialog alertDialog = builder.create();
        alertDialog.setCanceledOnTouchOutside(false);
        alertDialog.show();
    }

    //Experience SDK will be deprecated in PSDK 2.1.0
    @Deprecated
    private void configureExperienceSDK() {
        /*
          specify these parameters to integrate ExperienceSDK
          and pass them into PresenceSDK
          this will NOT crash if no ExperienceSDK lib provided
          presenceSDK.start() will check the availability of ExperienceSDK lib
          setSsoSigningKey() is optional, allows to use Pinless Feature
          to disable pin prompt within add, return, and upgrade buttons.
        */
        /*ExperienceConfiguration experienceConfiguration = new ExperienceConfiguration.Builder(
                BuildConfig.EXP_SUB_DOMAIN,
                BuildConfig.EXP_APP_ID,
                BuildConfig.EXP_APP_NAME,
                BuildConfig.EXP_APP_SOURCE)
                .setApiKey(BuildConfig.EXP_APP_API_KEY)
                .setSsoSigningKey(BuildConfig.EXP_SSO_SIGNING_KEY)
                .setApiVersion(BuildConfig.EXP_API_VERSION)
                .build();

        PresenceSDK.getPresenceSDK(getApplicationContext())
                .setExperienceConfiguration(experienceConfiguration);*/
    }


    @Override
    protected void onPause() {
        super.onPause();
        PresenceSDK.getPresenceSDK(getApplicationContext()).unregisterConfigListener(this);
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (!PresenceSDK.getPresenceSDK(this).isLoggedIn() && !mConfigSuccessFul) {
            mGettingStartedContainer.setVisibility(View.VISIBLE);
        } else {
            mGettingStartedContainer.setVisibility(View.GONE);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {

        MenuItem logoutMenu = menu.findItem(R.id.action_logout);
        MenuItem loginMenu = menu.findItem(R.id.action_login);
        if (PresenceSDK.getPresenceSDK(getApplicationContext()).isLoggedIn()) {
            logoutMenu.setVisible(true);
            loginMenu.setVisible(false);
        } else if (!mConfigSuccessFul) {
            loginMenu.setVisible(false);
            logoutMenu.setVisible(false);
        } else {
            logoutMenu.setVisible(false);
            loginMenu.setVisible(true);
        }

        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        int id = item.getItemId();
        if (id == R.id.action_logout) {
            invalidateOptionsMenu();
            PresenceSDK.getPresenceSDK(getApplicationContext()).logOut();
            configurePresenceSDK();

            return true;
        } else if (id == R.id.action_login) {

            mGettingStartedContainer.setVisibility(View.GONE);

            PresenceSDK.getPresenceSDK(getApplicationContext())
                    .start(PresenceSdkHostActivity.this, R.id.presenceSDKView, mSimpleLoginListener);

            return true;
        } else {
            return false;
        }
    }

}
